# sage_setup: distribution = sagemath-environment
r"""
Enumeration of all defined features
"""

# *****************************************************************************
#       Copyright (C) 2021-2023 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import itertools


def all_features():
    r"""
    Return an iterable of all features.

    EXAMPLES::

        sage: from sage.features.all import all_features
        sage: sorted(all_features(), key=lambda f: f.name)  # random
        [...Feature('sage.combinat')...]
    """
    import pkgutil
    import importlib
    import sage.features
    # Following https://packaging.python.org/guides/creating-and-discovering-plugins/#using-namespace-packages
    for finder, name, ispkg in pkgutil.iter_modules(sage.features.__path__, sage.features.__name__ + "."):
        module = importlib.import_module(name)
        try:
            af = module.all_features
        except AttributeError:
            pass
        else:
            if af != all_features:
                yield from af()


def module_feature(module_name):
    r"""
    Find a top-level :class:`Feature` that provides the Python module of the given ``module_name``.

    Only features known to :func:`all_features` are considered.

    INPUT:

    - ``module_name`` -- string

    OUTPUT: a :class:`Feature` or ``None``

    EXAMPLES::

        sage: from sage.features.all import module_feature
        sage: module_feature('sage.combinat.tableau')                                   # needs sage.combinat
        Feature('sage.combinat')
        sage: module_feature('sage.combinat.posets.poset')                              # needs sage.graphs
        Feature('sage.graphs')
        sage: module_feature('sage.schemes.toric.variety')                              # needs sage.geometry.polyhedron
        Feature('sage.geometry.polyhedron')
        sage: module_feature('scipy')                                                   # needs scipy
        Feature('scipy')
        sage: print(module_feature('sage.structure.element'))
        None
        sage: print(module_feature('sage.does_not_exist'))
        None
    """
    longest_prefix = ''
    longest_prefix_feature = None
    for feature in all_features():
        for joined in itertools.chain([feature], feature.joined_features()):
            if joined.name == module_name:
                return feature
            if (joined.name + '.').startswith(longest_prefix):
                if (module_name + '.').startswith(joined.name + '.'):
                    longest_prefix = feature.name + '.'
                    longest_prefix_feature = feature
    return longest_prefix_feature


def name_feature(name, toplevel=None):
    r"""
    Find a top-level :class:`Feature` that provides the top-level ``name``.

    Only features known to :func:`all_features` are considered.

    INPUT:

    - ``name`` -- string

    - ``toplevel`` -- a module or other namespace

    OUTPUT: a :class:`Feature` or ``None``

    EXAMPLES::

        sage: from sage.features.all import name_feature
        sage: name_feature('QuadraticField')                                            # needs sage.rings.number_field
        Feature('sage.rings.number_field')
        sage: name_feature('line')                                                      # needs sage.plot
        Feature('sage.plot')
        sage: print(name_feature('ZZ'))
        None
        sage: print(name_feature('does_not_exist'))
        None
    """
    if toplevel is None:
        try:
            import sage.all as toplevel
        except ImportError:
            return None
    try:
        obj = getattr(toplevel, name)
    except AttributeError:
        return None

    from sage.misc.dev_tools import find_object_modules

    for module, names in find_object_modules(obj).items():
        if name in names and (feature := module_feature(module)):
            return feature

    return None



# sage_setup: distribution = sagemath-environment
r"""
Check for igraph
"""

# ****************************************************************************
#       Copyright (C) 2021 Matthias Koeppe
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import PythonModule
from .join_feature import JoinFeature


class python_igraph(JoinFeature):
    r"""
    A :class:`sage.features.Feature` describing the presence of the
    Python package :ref:`igraph <spkg_python_igraph>`.

    EXAMPLES::

        sage: from sage.features.igraph import python_igraph
        sage: python_igraph().is_present()                    # optional - python_igraph
        FeatureTestResult('python_igraph', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.igraph import python_igraph
            sage: isinstance(python_igraph(), python_igraph)
            True
        """
        JoinFeature.__init__(self, 'python_igraph',
                             [PythonModule('igraph', spkg='python_igraph',
                                            url='http://igraph.org')])


def all_features():
    return [python_igraph()]



# sage_setup: distribution = sagemath-environment
# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.5.45'
date = '2025-07-04'
banner = 'passagemath version 10.5.45, Release Date: 2025-07-04'



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of QEPCAD
"""

# *****************************************************************************
#       Copyright (C) 2022 Marc Mezzarobba
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import subprocess
from . import Executable, FeatureTestResult, PythonModule
from .join_feature import JoinFeature


class Qepcad(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of :ref:`qepcad <spkg_qepcad>`.

    EXAMPLES::

        sage: from sage.features.qepcad import Qepcad
        sage: Qepcad().is_present()  # optional - qepcad
        FeatureTestResult('qepcad', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.qepcad import Qepcad
            sage: isinstance(Qepcad(), Qepcad)
            True
        """
        Executable.__init__(self, "qepcad", executable='qepcad',
                            spkg='qepcad')


def all_features():
    return [JoinFeature("qepcad",
                        (Qepcad(),
                         PythonModule('sage.interfaces.qepcad')),
                        spkg='sagemath_qepcad', type='optional')]



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of ``ecm`` or ``gmp-ecm``
"""
# ****************************************************************************
#       Copyright (C) 2032 Dima Pasechnik <dima@pasechnik.info>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import Executable, PythonModule
from .join_feature import JoinFeature
from sage.env import SAGE_ECMBIN


class Ecm(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of :ref:`GMP-ECM <spkg_ecm>`.

    EXAMPLES::

        sage: from sage.features.ecm import Ecm
        sage: Ecm().is_present()                                                        # needs ecm
        FeatureTestResult('ecm_executable', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.ecm import Ecm
            sage: isinstance(Ecm(), Ecm)
            True
        """
        Executable.__init__(self, name='ecm_executable', executable=SAGE_ECMBIN,
                            spkg='ecm', type='standard')


def all_features():
    return [JoinFeature("ecm",
                        (Ecm(),
                         PythonModule('sage.interfaces.ecm')),
                        spkg='sagemath_libecm', type='standard')]
